<?php
/**
 * Routes configuration.
 *
 * In this file, you set up routes to your controllers and their actions.
 * Routes are very important mechanism that allows you to freely connect
 * different URLs to chosen controllers and their actions (functions).
 *
 * It's loaded within the context of `Application::routes()` method which
 * receives a `RouteBuilder` instance `$routes` as method argument.
 *
 * CakePHP(tm) : Rapid Development Framework (https://cakephp.org)
 * Copyright (c) Cake Software Foundation, Inc. (https://cakefoundation.org)
 *
 * Licensed under The MIT License
 * For full copyright and license information, please see the LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright     Copyright (c) Cake Software Foundation, Inc. (https://cakefoundation.org)
 * @link          https://cakephp.org CakePHP(tm) Project
 * @license       https://opensource.org/licenses/mit-license.php MIT License
 */

use Cake\Http\ServerRequest;
use Cake\Routing\Route\DashedRoute;
use Cake\Routing\RouteBuilder;
use Cake\Routing\Router;

/*
 * The default class to use for all routes
 *
 * The following route classes are supplied with CakePHP and are appropriate
 * to set as the default:
 *
 * - Route
 * - InflectedRoute
 * - DashedRoute
 *
 * If no call is made to `Router::defaultRouteClass()`, the class used is
 * `Route` (`Cake\Routing\Route\Route`)
 *
 * Note that `Route` does not do any inflections on URLs which will result in
 * inconsistently cased URLs when used with `:plugin`, `:controller` and
 * `:action` markers.
 */
/** @var \Cake\Routing\RouteBuilder $routes */
$routes->setRouteClass(DashedRoute::class);

$routes->scope('/', function (RouteBuilder $builder) {
    /*
     * Here, we are connecting '/' (base path) to a controller called 'Pages',
     * its action called 'display', and we pass a param to select the view file
     * to use (in this case, templates/Pages/home.php)...
     */
    $builder->connect('/', ['controller' => 'AccessPoints', 'action' => 'index']);

    /*
     * Access Points - nested routes
     */
    $builder->connect('/access-points/{access_point_id}', ['controller' => 'AccessPoints', 'action' => 'view'])
        ->setPatterns(['access_point_id' => '[\w]{8}-[\w]{4}-[\w]{4}-[\w]{4}-[\w]{12}'])->setPass(['access_point_id']);
    $builder->connect('/access-points/{access_point_id}/edit', ['controller' => 'AccessPoints', 'action' => 'edit'])
        ->setPatterns(['access_point_id' => '[\w]{8}-[\w]{4}-[\w]{4}-[\w]{4}-[\w]{12}'])->setPass(['access_point_id']);
    $builder->connect('/access-points/{access_point_id}/{controller}', ['action' => 'index'])
        ->setPatterns(['access_point_id' => '[\w]{8}-[\w]{4}-[\w]{4}-[\w]{4}-[\w]{12}']);
    $builder->connect('/access-points/{access_point_id}/{controller}/{action}/*', [])
        ->setPatterns(['access_point_id' => '[\w]{8}-[\w]{4}-[\w]{4}-[\w]{4}-[\w]{12}']);

    /*
     * ...and connect the rest of 'Pages' controller's URLs.
     */
    $builder->connect('/pages/*', 'Pages::display');

    /*
     * Connect catchall routes for all controllers.
     *
     * The `fallbacks` method is a shortcut for
     *
     * ```
     * $builder->connect('/:controller', ['action' => 'index']);
     * $builder->connect('/:controller/:action/*', []);
     * ```
     *
     * You can remove these routes once you've connected the
     * routes you want in your application.
     */
    $builder->fallbacks();
});

/*
 * If you need a different set of middleware or none at all,
 * open new scope and define routes there.
 *
 * ```
 * $routes->scope('/api', function (RouteBuilder $builder) {
 *     // No $builder->applyMiddleware() here.
 *
 *     // Parse specified extensions from URLs
 *     // $builder->setExtensions(['json', 'xml']);
 *
 *     // Connect API actions here.
 * });
 * ```
 */

$routes->prefix('Api', function (RouteBuilder $builder) {
    $builder->setExtensions(['json']);

    $builder->resources('AccessPoints');
    $builder->resources('IpAddressRanges', [
        'map' => [
            'search' => [
                'action' => 'search',
                'method' => 'GET',
            ],
        ],
    ]);
    $builder->resources('RouterosDevices', [
        'map' => [
            'search' => [
                'action' => 'search',
                'method' => 'GET',
            ],
        ],
    ]);
});

//apply URL filters only if not called from console
if (!(php_sapi_name() == 'cli')) {
    Router::addUrlFilter(function (array $params, ServerRequest $request) {
        //persistent win-link parameter
        if ($request->getQuery('win-link') == 'true') {
            $params['?']['win-link'] = 'true';
        }

        //controllers related to access points
        if (
            (isset($params['controller']) && in_array($params['controller'], [
                'AccessPointContacts',
                'ElectricityMeterReadings',
                'IpAddressRanges',
                'PowerSupplies',
                'RadioUnits',
                'RouterosDevices',
            ]))
            || (!isset($params['controller']) && in_array($request->getParam('controller'), [
                'AccessPointContacts',
                'ElectricityMeterReadings',
                'IpAddressRanges',
                'PowerSupplies',
                'RadioUnits',
                'RouterosDevices',
            ]))
        ) {
            //inject access_point_id
            if ($request->getParam('access_point_id') && !isset($params['access_point_id'])) {
                $params['access_point_id'] = $request->getParam('access_point_id');
            }
        }

        return $params;
    });
}
